<?php


namespace com\cminds\maplocations\controller;


use com\cminds\maplocations\model\Location;
use com\cminds\maplocations\model\Route;

class ApiController extends Controller {

	static function bootstrap() {
		parent::bootstrap();
		// We need to check settings before adding hooks
		//add_action('plugins_loaded', array(get_called_class(), 'plugins_loaded'), 300);
		add_action( 'rest_api_init', function () {
			register_rest_route( 'cmloc/v1', '/add_location', array(
				'methods'  => \WP_REST_Server::ALLMETHODS,
				'callback' => array( self::class, 'action' )

			) );
		}, 5 );
	}


	static function getUserIP() {
		// Get real visitor IP behind CloudFlare network
		if ( isset( $_SERVER["HTTP_CF_CONNECTING_IP"] ) ) {
			$_SERVER['REMOTE_ADDR']    = $_SERVER["HTTP_CF_CONNECTING_IP"];
			$_SERVER['HTTP_CLIENT_IP'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
		}
		$client  = @$_SERVER['HTTP_CLIENT_IP'];
		$forward = @$_SERVER['HTTP_X_FORWARDED_FOR'];
		$remote  = $_SERVER['REMOTE_ADDR'];

		if ( filter_var( $client, FILTER_VALIDATE_IP ) ) {
			$ip = $client;
		} elseif ( filter_var( $forward, FILTER_VALIDATE_IP ) ) {
			$ip = $forward;
		} else {
			$ip = $remote;
		}

		return $ip;
	}


	static function action( \WP_REST_Request $request ) {

		$result = array_map( 'trim', explode( ',', trim( get_option( 'cmloc_rest_api_ip' ) ) ) );

		if ( ! get_option( 'cmloc_api_enable' ) ) {
			die;
		}
		if ( ! in_array( self::getUserIP(), $result ) ) {
			if ( $result[0] != '' ) {
				die;
			}

		}


		$data = shortcode_atts( array(
			'name'                   => '',
			'description'            => '',
			'images'                 => '',
			'status'                 => 'publish',
			'use-minor-length-units' => 1,
			'icon'                   => '',
			'icon_size'              => '',
		), $_GET );


		if ( ! isset( $_GET['key']{1} ) ) {

			return [ 'status' => 'error', 'message' => esc_html__( 'You need to provide API key', 'cmloc' ) ];
		}
		$API_KEY = trim( get_option( 'cmloc_api_key' ) );
		if ( ! isset( $API_KEY{1} ) ) {
			return [
				'status'  => 'error',
				'message' => esc_html__( 'You need to generate API key in Settings -> Setup', 'cmloc' )
			];

		} elseif ( trim( $_GET['key'] ) != $API_KEY ) {
			return [ 'status' => 'error', 'message' => esc_html__( 'You need to provide API key', 'cmloc' ) ];

		}
		if ( ! isset( $data['name']{1} ) ) {
			return [ 'status' => 'error', 'message' => esc_html__( 'You need to provide name', 'cmloc' ) ];
		}
		if ( ! isset( $_GET['LocLAT']{1} ) ) {
			return [ 'status' => 'error', 'message' => esc_html__( 'You need to provide LocLAT', 'cmloc' ) ];
		}
		if ( ! isset( $_GET['LocLONG']{1} ) ) {
			return [ 'status' => 'error', 'message' => esc_html__( 'You need to provide LocLONG', 'cmloc' ) ];
		}


		if ( isset( $_GET['id'] ) ) {
			$route = Route::getInstance( $_GET['id'] );
		} else {
			$route = new Route();
		}

		$valid = apply_filters( 'cmloc_dashboard_route_save_validation', true, $route, $data );
		if ( ! $valid ) {
			return;
		}


		if ( get_option( 'cmloc_api_prevent_duplicate' ) && self::locationExists( $data['name'] ) ) {
			return [
				'status'  => 'error',
				'message' => esc_html__( 'location with this name already exists', 'cmloc' )
			];

		}

		$route->setTitle( $data['name'] );

		$route->setContent( $data['description'] );
		$route->setStatus( $data['status'] );
		$route->setAuthor( get_current_user_id() );

		$id = $route->save();

		$status  = 'success';
		$message = '';
		if ( $id ) {

			do_action( 'cmloc_clear_cache' );

			$route->setImages( $data['images'] );
			$route->setIcon( $data['icon'] );
			$route->setIconSize( $data['icon_size'] );

			if ( isset( $_GET['LocCAT'] ) ) {


				$cat  = sanitize_text_field( trim( $_GET['LocCAT'] ?? '' ) );
				$term = get_term_by( 'name', $cat, 'cmloc_category' );

				if ( isset( $term->term_id ) ) {
					$route->setCategories( [ $term->term_id ] );
				} elseif ( get_option( 'cmloc_api_create_category' ) ) {
					$insert_data = wp_insert_term(
						$cat,  //
						'cmloc_category' //
					);
					if ( ! is_wp_error( $insert_data ) ) {
						$route->setCategories( [ $insert_data['term_id'] ] );
					}


				}

			}


			self::processSaveLocations( $route );

			$message = esc_html__( 'successfully added', 'cmloc' );

		} else {
			$status  = 'error';
			$message = esc_html__( 'some error occurred', 'cmloc' );

		}

		return [ 'status' => $status, 'message' => $message, 'id' => $id ];


	}

	public static function processSaveLocations( $route ) {
		$oldLocationsIds = $route->getLocationsIds();
		$newLocationsIds = array();

		if ( ! empty( $_GET['LocLAT'] ) && ! empty( $_GET['LocLONG'] ) ) {

			$location = new Location( array(
				'post_parent'    => $route->getId(),
				'post_author'    => get_current_user_id(),
				'post_type'      => Location::POST_TYPE,
				'post_status'    => 'inherit',
				'ping_status'    => 'closed',
				'comment_status' => 'closed',
			) );

			$location->setTitle( $_GET['loc-name'] ?? '' );
			$location->setContent( $_GET['loc-description'] ?? '' );
			$location->setMenuOrder( 1 );
			$id = $location->save();

			$newLocationsIds[] = $id;
			if ( $id ) {
				$location->setLat( $_GET['LocLAT'] );
				$location->setLong( $_GET['LocLONG'] );
				$location->setLatLong( $_GET['LocLAT'] . ',' . $_GET['LocLONG'] );
				$location->setLocationType( $_GET['loc-type'] ?? '' );
				$location->setMarkerAlt( $_GET['loc-marker-alt'] ?? '' );
				$location->setAddress( $_GET['loc-address'] ?? '' );
				$location->setPostalCode( $_GET['loc-postal-code'] ?? '' );
				$location->setImages( $_GET['loc-images'] ?? '' );
				$location->setPhoneNumber( $_GET['loc-phone-number'] ?? '' );
				$location->setWebsite( $_GET['loc-website'] ?? '' );
				$location->setEmail( $_GET['loc-email'] ?? '' );
				$location->setUrl( $_GET['loc-url'] ?? '' );
				do_action( 'cmloc_location_after_save', $location, 1 );
			}


		}


		// Remove unused locations
		$toRemove = array_diff( $oldLocationsIds, array_filter( $newLocationsIds ) );
		foreach ( $toRemove as $id ) {
			wp_delete_post( $id, $force = true );
		}


		$route->updateLocationsAltitudes();
	}


	static public function locationExists( $title, $type = '' ) {
		global $wpdb;
		$type = 'cmloc_object';

		$post_title = wp_unslash( sanitize_post_field( 'post_title', $title, 0, 'db' ) );
		$post_type  = wp_unslash( sanitize_post_field( 'post_type', $type, 0, 'db' ) );

		$query = "SELECT ID FROM $wpdb->posts WHERE 1=1";
		$args  = array();


		if ( ! empty( $title ) ) {
			$query  .= ' AND post_title = %s';
			$args[] = $post_title;
		}


		if ( ! empty( $type ) ) {
			$query  .= ' AND post_type = %s';
			$args[] = $post_type;
		}

		if ( ! empty( $args ) ) {
			return (int) $wpdb->get_var( $wpdb->prepare( $query, $args ) );
		}

		return 0;
	}

}