<?php
namespace com\cminds\maplocations\controller;

use com\cminds\maplocations\model\Expert;
use com\cminds\maplocations\helper\GoogleMapsIcons;
use com\cminds\maplocations\model\ExpertCategory;
use com\cminds\maplocations\shortcode\ExpertShortcode;
use com\cminds\maplocations\model\Settings;
use com\cminds\maplocations\model\Labels;
use com\cminds\maplocations\model\Route;
use com\cminds\maplocations\model\Location;

class ExpertController extends Controller {

    const ICON_NONCE_NAME           = 'cmloc_expert_category_icon_nonce';
    const OFTEN_CRON_SCHEDULE_NAME  = 'cmloc_exp_geoloc';
    const RARELY_CRON_SCHEDULE_NAME = 'hourly';
    const CRON_ACTION               = 'cmloc_geocode_locations';
    const OPTION_ED_INDEX_SHOW_MAP  = 'cmloc_ed_index_show_map';

    protected static $filters = array(
        'cron_schedules',
        'cmed_index_after_filters' => array( 'args' => 2 ),
        'cmed_options_config',
        'cmdb_index_additional_filters' => array( 'args' => 2 ),
        'posts_search'             => array( 'args' => 2 ),
    );

    protected static $actions = array(
        'init',
        'cm-expert-category_edit_form_fields'          => array( 'method' => 'formFields', 'args' => 1 ),
        'cm-expert-category_add_form_fields'           => array( 'method' => 'formFields', 'args' => 1 ),
        'edited_cm-expert-category'                    => array( 'method' => 'categoryAfterSave', 'args' => 2 ),
        'created_cm-expert-category'                   => array( 'method' => 'categoryAfterSave', 'args' => 2 ),
        'pre_post_update'                                => array( 'args' => 2, 'method' => 'beforePostSave' ),
        'cmed_after_expert_admin_form_address_section' => array( 'args' => 1, 'method' => 'expertAdminFormAddressSection' ),
        'save_post'                                      => array( 'args' => 1, 'method' => 'afterPostSave' ),
        self::CRON_ACTION,
    );

    static $ajax = array(
		'cmloc_expert_map_filter'
	);

	static function expertAdminFormAddressSection( $post ) {
    	$expert = Expert::getInstance( $post->ID );
    	$lat = $expert->getLat();
    	$lng = $expert->getLong();
    	$manual_marker = $expert->getManualMarker();
		echo static::loadBackendView( 'coordinates', compact( 'expert', 'lat', 'lng', 'manual_marker' ) );
    }

    static function init() {
		//self::cmloc_geocode_locations();
		//var_dump(wp_get_schedule(ExpertController::CRON_ACTION));exit;
        if ( wp_get_schedule( ExpertController::CRON_ACTION ) === false ) {
            ExpertController::scheduleEvent();
        }
    }

    static function scheduleEvent( $schedule = self::OFTEN_CRON_SCHEDULE_NAME ) {
        static::removeScheduledEvent();
        wp_schedule_event( time(), $schedule, ExpertController::CRON_ACTION );
    }

    static function removeScheduledEvent() {
        wp_clear_scheduled_hook( static::CRON_ACTION );
    }

    static function cmloc_expert_map_filter() {
        echo ExpertShortcode::shortcode( $_POST[ 'atts' ] );
        exit;
    }

     static function formFields( $term = null ) {

        if ( !empty( $term ) AND is_object( $term ) AND $category = ExpertCategory::getInstance( $term ) ) {
            $currentIcon = $category->getIcon();
        } else {
            $term        = null;
            $currentIcon = null;
        }

        $nonceField = static::ICON_NONCE_NAME;
        $nonce      = wp_create_nonce( static::ICON_NONCE_NAME );
        $icons      = GoogleMapsIcons::getAll();

        wp_enqueue_script( 'cmloc-backend' );

        echo static::loadBackendView( 'form-icon', compact( 'term', 'icons', 'currentIcon', 'nonce', 'nonceField' ) );
    }

    static function categoryAfterSave( $term_id, $term_taxonomy_id = null ) {
        // Get category object
        $category =ExpertCategory::getInstance( $term_id );
        if ( empty( $category ) )
            return;

        if ( isset( $_POST[ static::ICON_NONCE_NAME ] ) AND wp_verify_nonce( $_POST[ static::ICON_NONCE_NAME ], static::ICON_NONCE_NAME )
        AND ! empty( $_POST[ static::ICON_NONCE_NAME ] ) ) {
            $category->setIcon( $_POST[ 'cmloc_category_icon' ] );
        }
    }


     static function cmloc_geocode_locations() {
        global $wpdb;

        Expert::cleanLocationForEmptyAddresses();
        $ids = Expert::getIdsNeedGeolocation();
        foreach ( $ids as $id ) {
            $bus = Expert::getInstance( $id );
            if ( $bus ) {
                $bus->setLocationFromAddress();
            }
            Expert::clearInstances();
        }

        // Reschedule if needed
        $count = count( $ids );
        if ( $count == 0 ) { // all remaining posts has been already geolocalized
            if ( wp_get_schedule( static::CRON_ACTION ) != static::RARELY_CRON_SCHEDULE_NAME ) {
                // reschedule the cron to run less frequently
                static::scheduleEvent( static::RARELY_CRON_SCHEDULE_NAME );
            }
        } else if ( $count > 60 ) { // more posts remaining
            if ( wp_get_schedule( static::CRON_ACTION ) != static::OFTEN_CRON_SCHEDULE_NAME ) {
                // reschedule cron to run more frequently
                static::scheduleEvent( static::OFTEN_CRON_SCHEDULE_NAME );
            }
        }
    }

    static function beforePostSave( $postId, $data ) {
        if ( $bus = Expert::getInstance( $postId ) ) {
            if ( (isset( $_POST[ 'cmed_address' ] ) AND $bus->getAddress() != $_POST[ 'cmed_address' ])
            OR ( isset( $_POST[ 'cmed_cityTown' ] ) AND $bus->getCityTown() != $_POST[ 'cmed_cityTown' ])
            OR ( isset( $_POST[ 'cmed_region' ] ) AND $bus->getRegion() != $_POST[ 'cmed_region' ])
            OR ( isset( $_POST[ 'cmed_stateCounty' ] ) AND $bus->getstateCounty() != $_POST[ 'cmed_stateCounty' ])
            OR ( isset( $_POST[ 'cmed_county' ] ) AND $bus->getCountry() != $_POST[ 'cmed_county' ])
            OR ( isset( $_POST[ 'cmed_postalcode' ] ) AND $bus->getPostalCode() != $_POST[ 'cmed_postalcode' ])
	        OR !( isset( $_POST[ Expert::META_ED_MANUAL_MARKER ] ) AND $_POST[ Expert::META_ED_MANUAL_MARKER ] === '1' AND
	              $bus->getManualMarker() == '1' )
            ) {
                $bus->setGeocodeUpToDate( false );
            }
        }
    }

    static function afterPostSave($postId) {
        if($bus = Expert::getInstance($postId)) {
        	if(isset($_POST[Expert::META_ED_MANUAL_MARKER]) && $_POST[Expert::META_ED_MANUAL_MARKER] === '1') {
				$bus->setManualMarker(isset($_POST[Expert::META_ED_MANUAL_MARKER]) ? $_POST[Expert::META_ED_MANUAL_MARKER] : '0');
        	    $bus->setLat(isset($_POST[Expert::META_LAT]) ? $_POST[Expert::META_LAT] : '');
        	    $bus->setLong(isset($_POST[Expert::META_LONG]) ? $_POST[Expert::META_LONG] : '');
				if(isset($_POST[Expert::META_LAT]) && isset($_POST[Expert::META_LONG])) {
        			$bus->setLatLong($_POST[Expert::META_LAT].','.$_POST[Expert::META_LONG]);
				}
	        } else {
				//$bus->setManualMarker(isset($_POST[Expert::META_ED_MANUAL_MARKER]) ? $_POST[Expert::META_ED_MANUAL_MARKER] : '0');
		        //$bus->setLocationFromAddress();
				if(isset($_POST['post_ID']) && isset($_POST[Expert::META_ED_MANUAL_MARKER]) && $_POST[Expert::META_ED_MANUAL_MARKER] === '0') {
					$bus->setManualMarker(isset($_POST[Expert::META_ED_MANUAL_MARKER]) ? $_POST[Expert::META_ED_MANUAL_MARKER] : '0');
				}
				if(isset($_POST['post_ID']) && isset($_POST[Expert::META_LAT])) {
					$bus->setLat(isset($_POST[Expert::META_LAT]) ? $_POST[Expert::META_LAT] : '');
				}
				if(isset($_POST['post_ID']) && isset($_POST[Expert::META_LONG])) {
					$bus->setLong(isset($_POST[Expert::META_LONG]) ? $_POST[Expert::META_LONG] : '');
				}
				if(isset($_POST['post_ID']) && isset($_POST[Expert::META_LAT]) && isset($_POST[Expert::META_LONG])) {
					$bus->setLatLong($_POST[Expert::META_LAT].','.$_POST[Expert::META_LONG]);
				}
	        }
        }
    }

    static function cron_schedules( $interval ) {
        $interval[ static::OFTEN_CRON_SCHEDULE_NAME ] = array( 'interval' => 2 * 60, 'display' => 'Once 2 minutes for CM Locations' );
        return $interval;
    }

    static function cmed_options_config( $options ) {

        $options[ static::OPTION_ED_INDEX_SHOW_MAP ] = array(
            'type'        => 'bool',
            'default'     => 0,
            'category'    => 'index',
            'subcategory' => 'shortcode',
            'title'       => 'Show CM Locations Map',
            'desc'        => 'Display the CM Map Locations shortcode on the index page.',
        );

        return $options;
    }

    static function cmed_index_after_filters( $content, \WP_Query $query ) {
        if ( !get_option( static::OPTION_ED_INDEX_SHOW_MAP ) ) {
            return $content;
        }
        $ids = array();
        foreach ( $query->posts as $post ) {
            $ids[] = $post->ID;
        }
        $shortcode = ExpertShortcode::shortcode( array(
            'categoryfilter' => 0,
            'searchbar'      => 0,
            'ids'            => $ids,
        ) );
        return $content . self::loadFrontendView( 'index-after-filters', compact( 'shortcode' ) );
    }

    /*
     * ZIP Radius
     */

    static function cmdb_index_additional_filters( $content, $atts ) {
        if ( !Settings::getOption( Settings::OPTION_INDEX_ZIP_RADIUS_FILTER_ENABLE ) ) {
            return $content;
        }
        $radiusOptions = array();
        $unitLen       = Settings::getOption( Settings::OPTION_UNIT_LENGTH );
        $start         = Settings::getOption( Settings::OPTION_INDEX_ZIP_RADIUS_MIN );
        $max           = Settings::getOption( Settings::OPTION_INDEX_ZIP_RADIUS_MAX );
        $defaultRadius = Settings::getOption( Settings::OPTION_INDEX_ZIP_RADIUS_DEFAULT );
        $step          = Settings::getOption( Settings::OPTION_INDEX_ZIP_RADIUS_STEP );
        for ( $val = $start; $val <= $max; $val += $step ) {
            if ( $unitLen == Settings::UNIT_FEET ) {
                $valMeters = $val * Settings::FEET_IN_MILE * Settings::FEET_TO_METER;
            } else {
                $valMeters = $val * 1000;
            }
            $valKm                   = round( $valMeters / 1000 );
            $radiusOptions[ $valKm ] = $val . ' ' . ($unitLen == Settings::UNIT_METERS ? Labels::getLocalized( 'length_km' ) : Labels::getLocalized( 'length_miles' ));
        }

        $radiusValue = filter_input( INPUT_GET, 'zipradius' );
        if ( empty( $radiusValue ) ) {
            $radiusValue = ($unitLen == Settings::UNIT_METERS ? $defaultRadius : $defaultRadius * Settings::FEET_IN_MILE * Settings::FEET_TO_METER);
        }

        $zipcodeValue = filter_input( INPUT_GET, 'zipcode' );

        return $content . self::loadFrontendView( 'zip-filter', compact( 'radiusOptions', 'zipcodeValue', 'radiusValue' ) );
    }

    static function posts_search( $search, \WP_Query $query ) {
        if ( Settings::getOption( Settings::OPTION_INDEX_ZIP_RADIUS_FILTER_ENABLE ) AND Expert::isPostType( $query ) ) {
            $zipcode  = filter_input( INPUT_GET, 'zipcode' );
            $radiusKm = filter_input( INPUT_GET, 'zipradius' );
            if ( $zipcode AND $radiusKm ) {

                $radiusMeters = $radiusKm * 1000;
                $radiusMiles  = $radiusMeters / Settings::FEET_TO_METER / Settings::FEET_IN_MILE;

                $coords = Route::findLocationByAddress( $zipcode . ' ' . Settings::getOption( Settings::OPTION_INDEX_ZIP_RADIUS_COUNTRY ) );
				//var_dump($coords);exit;
                if ( !empty( $coords ) ) {

                    $query->cmlocRadiusMeters = $radiusMeters;
                    $query->cmlocZipcode      = $zipcode;
                    $query->cmlocCoords       = $coords;

                    $sql = '(
				          acos(sin(cmloc_ziplat.meta_value * 0.0175) * sin(' . $coords[ 0 ] . ' * 0.0175)
				               + cos(cmloc_ziplat.meta_value * 0.0175) * cos(' . $coords[ 0 ] . ' * 0.0175) *
				                 cos((' . $coords[ 1 ] . ' * 0.0175) - (cmloc_ziplng.meta_value * 0.0175))
				              ) * 3959 <= ' . $radiusMiles . '
				      )';

                    $search .= ' AND ' . $sql;

                    // Add required joins
                    add_filter( 'posts_join', array( __CLASS__, 'posts_search_join' ), 10, 2 );
                }
            }
        }
        return $search;
    }

    static function posts_search_join( $join, \WP_Query $query ) {
        global $wpdb;
        //Additional joins to search by address and postal code
		//$join .= PHP_EOL . "JOIN $wpdb->posts cmloc_ziploc ON cmloc_ziploc.post_parent = $wpdb->posts.ID";
        $join .= PHP_EOL . $wpdb->prepare( "JOIN $wpdb->postmeta cmloc_ziplat ON cmloc_ziplat.post_id = $wpdb->posts.ID AND cmloc_ziplat.meta_key = %s", Location::META_LAT );
        $join .= PHP_EOL . $wpdb->prepare( "JOIN $wpdb->postmeta cmloc_ziplng ON cmloc_ziplng.post_id = $wpdb->posts.ID AND cmloc_ziplng.meta_key = %s", Location::META_LONG );
        $join .= PHP_EOL;
        remove_filter( 'posts_join', array( __CLASS__, 'posts_search_join' ), 10 );
        return $join;
    }

}