<?php
namespace com\cminds\maplocations\controller;

use com\cminds\maplocations\model\Route;
use com\cminds\maplocations\model\Location;
use com\cminds\maplocations\App;
use com\cminds\maplocations\model\Settings;
use com\cminds\maplocations\model\Labels;

class ProController extends Controller {

	public static function bootstrap() {
		self::$ajax[] = 'cmlocGenerateKey';
		parent::bootstrap(); // TODO: Change the autogenerated stub
	}

	public static function cmlocGenerateKey(){
		wp_die(wp_generate_uuid4());

	}

	protected static $filters = array(
		'cmloc_options_config',
		'cmloc_settings_pages',
		'cmloc_route_icon_url' => array('args' => 2),
	);

	protected static $actions = array(
		array('name' => 'admin_menu', 'priority' => 96),
		array('name' => 'cmloc_labels_init', 'priority' => 10),
		'cmloc_load_assets_frontend',
		'cmloc_single_location_before_images' => array('args' => 1),
		'cmloc_route_editor_middle' => array('args' => 1),
	);

	static function admin_menu() {
		//add_submenu_page(App::PREFIX, 'About ' . App::getPluginName(), 'About', 'manage_options', self::getMenuSlug('about'), array(get_called_class(), 'about'));
		//add_submenu_page(App::PREFIX, App::getPluginName() . ' User Guide', 'User Guide', 'manage_options', self::getMenuSlug('user-guide'),
		//array(get_called_class(), 'userGuide'));
	}
	
	static function getMenuSlug($slug) {
		return App::PREFIX . '-' . $slug;
	}

	static function about() {
		echo self::loadView('backend/template', array(
			'title' => 'About ' . App::getPluginName(),
			'nav' => self::getBackendNav(),
			'content' => self::loadBackendView('about', array(
				'iframeURL' => SettingsController::PAGE_ABOUT_URL,
			)) . SettingsController::getSectionExperts(),
		));
	}
	
	static function userGuide() {
		echo self::loadView('backend/template', array(
			'title' => App::getPluginName() . ' User Guide',
			'nav' => self::getBackendNav(),
			'content' => self::loadBackendView('about', array(
				'iframeURL' => SettingsController::PAGE_USER_GUIDE_URL,
			)) . SettingsController::getSectionExperts(),
		));
	}

	static function cmloc_labels_init() {
		Labels::loadLabelFile(App::path('asset/labels/pro.tsv'));
	}
	
	static function cmloc_options_config($config) {
		return array_merge($config, array(
			Settings::OPTION_DONT_EMBED_GOOGLE_MAPS_JS_API => array(
				'type' => Settings::TYPE_BOOL,
				'category' => 'setup',
				'subcategory' => 'api',
				'default' => 0,
				'title' => 'Disable embeding Google Maps JS API',
				'desc' => 'Enabling this option can solve some conflicts with other plugins or themes that also includes the Google Maps API on each page, eg. Geodirectory.',
			),
			Settings::OPTION_INDEX_ORDERBY => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					Settings::ORDERBY_TITLE => 'title',
					Settings::ORDERBY_CREATED => 'created date',
					Settings::ORDERBY_VIEWS => 'views',
					Settings::ORDERBY_RATING => 'rating',
					Settings::ORDERBY_PROXIMITY => 'user location proximity',
				),
				'default' => Settings::DEFAULT_INDEX_ORDERBY,
				'category' => 'index',
				'subcategory' => 'pagination',
				'title' => 'Order locations by',
				'desc' => 'Notice: user location proximity uses web browser\'s geolocation API which requires using SSL (https).',
			),
			Settings::OPTION_INDEX_ORDER => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					Settings::ORDER_ASC => 'ascending',
					Settings::ORDER_DESC => 'descending',
				),
				'default' => Settings::DEFAULT_INDEX_ORDER,
				'category' => 'index',
				'subcategory' => 'pagination',
				'title' => 'Sorting order',
			),
			Settings::OPTION_INDEX_SEARCH_MODE => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'match' => 'Exact Match',
					'nearest' => 'Nearest',
				),
				'default' => 'match',
				'category' => 'index',
				'subcategory' => 'pagination',
				'title' => 'Search mode',
			),
			Settings::OPTION_INDEX_SEARCH_NEAREST_RADIUS => array(
				'type' => Settings::TYPE_INT,
				'default' => '50',
				'category' => 'index',
				'subcategory' => 'pagination',
				'title' => 'Nearest radius value',
				'desc' => 'Notice: radius will work with search mode nearest only.'
			),
			Settings::OPTION_INDEX_MAP_ENABLE => array(
				'type' => Settings::TYPE_BOOL,
				'default' => true,
				'category' => 'index',
				'subcategory' => 'map',
				'title' => 'Enable map',
				'desc' => 'If enabled, then map will show on index page.'
			),
			Settings::OPTION_INDEX_MAP_MARKER_CLUSTERING_ENABLE => array(
				'type' => Settings::TYPE_BOOL,
				'default' => true,
				'category' => 'index',
				'subcategory' => 'map',
				'title' => 'Enable marker clustering',
				'desc' => 'If enabled, the specific locations\' markers that are overlaping on the map will be clustered in a single icon with a number of locations. '
				. 'The single marker will be showed up after clicking on the cluster icon or zooming in the area.'
			),
			Settings::OPTION_INDEX_MAP_INFOWINDOW_MARKER_CLUSTERING_ENABLE => array(
				'type' => Settings::TYPE_BOOL,
				'default' => false,
				'category' => 'index',
				'subcategory' => 'map',
				'title' => 'Enable info window on marker clustering',
				'desc' => 'If enabled, the info window will open on click the marker cluster.'
			),
			Settings::OPTION_UNIT_TEMPERATURE => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(Settings::UNIT_TEMP_C => 'Celsius', Settings::UNIT_TEMP_F => 'Fahrenheit'),
				'default' => Settings::UNIT_TEMP_C,
				'category' => 'general',
				'subcategory' => 'units',
				'title' => 'Temperature units',
				'desc' => 'Used to display the weather.',
			),
			Settings::OPTION_TEMPLATE_OVERRIDE_DIR => array(
				'type' => Settings::TYPE_STRING,
				'category' => 'general',
				'subcategory' => 'template',
				'title' => 'Override templates with files from a directory',
				'desc' => 'Enter the directory path relative to the wp-content directory to override the plugin\'s view templates.',
			),
			Settings::OPTION_INDEX_PAGE_ENABLE => array(
				'type' => Settings::TYPE_BOOL,
				'default' => 1,
				'category' => 'general',
				'subcategory' => 'template',
				'title' => 'Enable default location index page',
				'desc' => 'If enabled, then will work default location index page',
			),
			Settings::OPTION_DASHBOARD_PAGE_ENABLE => array(
				'type' => Settings::TYPE_BOOL,
				'default' => 1,
				'category' => 'general',
				'subcategory' => 'template',
				'title' => 'Enable default user dashboard page',
				'desc' => 'If enabled, then will work default user dashboard page',
			),
			Settings::OPTION_SINGLE_PAGE_ENABLE => array(
				'type' => Settings::TYPE_BOOL,
				'default' => 1,
				'category' => 'general',
				'subcategory' => 'template',
				'title' => 'Enable default location\'s page',
				'desc' => 'If enabled, then will work default location\'s page',
			),

			Settings::OPTION_OPENWEATHERMAP_API_KEY => array(
				'type' => Settings::TYPE_STRING,
				'category' => 'setup',
				'subcategory' => 'api',
				'title' => 'OpenWeatherMap.org API Key',
				'desc' => 'Enter the OpenWeatherMap.org API key.<br /><a target="_blank" '. 'href="https://openweathermap.org/appid">Get the API key from here</a>.',
			),
			'cmloc_api_enable' => array(
				'type' => Settings::TYPE_BOOL,
				'category' => 'setup',
				'subcategory' => 'rest',
				'title' => 'Enable REST API',
				'desc' => 'Enable to add locations from external services via API <br>
<a target="_blank" href="https://creativeminds.helpscoutdocs.com/article/2559-cm-map-locations-cmml-adding-locations-from-external-services-rest-api">Learn more about using REST API with Locations</a>
',

			),'cmloc_api_key' => array(
				'type' => Settings::TYPE_STRING,
				'category' => 'setup',
				'subcategory' => 'rest',
				'title' => 'API key for adding location via REST API',
				'desc' => 'Enter the API key or generate it. You should add this key in the settings of the external service used to manage locations<br /><br />
<button class="button button-primary" id="cms-generate-api-key" type="button">Generate Key</button>

<br>
Use this URL template to add locations. Replace the underlined elements with the relevant ones <br>
<u>http://your-site.com</u>/wp-json/cmloc/v1/add_location?description=<u>yourdescription</u>&name=<u>your  location name</u>&LocCAT=<u>category</u>&LocLAT=<u>50.401699</u>&LocLONG=<u>30.252512</u>&key=<u>abc123</u>
<br>

',
			),

			'cmloc_rest_api_ip' => array(
				'type' => Settings::TYPE_STRING,
				'category' => 'setup',
				'subcategory' => 'rest',
				'title' => 'Enter the IP from which you can connect:',
				'desc' => 'Enter the IP from which you can connect to the API. Add multiples by separating them with comas (e.g. 145.0.0.5,145.0.0.6).
Leave this field empty to allow connections from any IP',
			),

			'cmloc_api_create_category' => array(
				'type' => Settings::TYPE_BOOL,
				'category' => 'setup',
				'subcategory' => 'rest',
				'title' => 'Create category if it doesn\'t exist:',
				'desc' => 'Choose how to handle a category that is specified in the URL but doesn\'t exist. If enabled, the category will be created. If disabled, it will be disregarded.',

			),
			'cmloc_api_prevent_duplicate' => array(
				'type' => Settings::TYPE_BOOL,
				'category' => 'setup',
				'subcategory' => 'rest',
				'title' => 'Don\'t add locations with the same name:',
				'desc' => 'Enable to not create locations with the same name. Disable to allow locations with the same name',

			),

			Settings::OPTION_INDEX_LOCATIONS_LIST_LAYOUT => array(
				'type' => Settings::TYPE_RADIO,
				'default' => Settings::INDEX_LIST_BOTTOM,
				'options' => array(
					Settings::INDEX_LIST_BOTTOM => 'Bottom',
					Settings::INDEX_LIST_BOTTOM_CONDENSED => 'Bottom condensed',
					Settings::INDEX_LIST_LEFT => 'Left',
					Settings::INDEX_LIST_RIGHT => 'Right',
				),
				'category' => 'index',
				'subcategory' => 'layout',
				'title' => 'Where to show locations list',
				'desc' => 'Choose where to display the locations list on the index page.',
			),
			Settings::OPTION_INDEX_MENU_ENABLE => array(
				'type' => Settings::TYPE_BOOL,
				'default' => false,
				'category' => 'index',
				'subcategory' => 'layout',
				'title' => 'Enable menu',
				'desc' => 'If enabled, then location menu will show above map.'
			),
			Settings::OPTION_INDEX_MAP_LABEL_TYPE => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					Settings::LABEL_TYPE_SHOW_BELOW => 'Show always below the marker',
					Settings::LABEL_TYPE_TOOLTIP => 'Show tooltip on mouse over',
					Settings::LABEL_TYPE_NONE => 'Do not show label',
				),
				'default' => Settings::LABEL_TYPE_SHOW_BELOW,
				'category' => 'index',
				'subcategory' => 'map',
				'title' => 'Marker label type',
				'desc' => 'Choose the label type with the marker\'s name on the location\'s map.',
			),
			
			Settings::OPTION_MAP_SHOW_PLACES => array(
				'type' => Settings::TYPE_BOOL,
				'default' => 1,
				'category' => 'general',
				'subcategory' => 'map',
				'title' => 'Show Google Places on the map',
				'desc' => 'If enabled, the places from Google Places API (such as restaurants, gas stations) will be shown on the map.',
			),
			
			// Location Form
			Settings::OPTION_LOCATION_FORM_DESCRIPTION => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Optional',
					'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Description',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_STATUS => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Visible',
					//'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Status',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_CATEGORY => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Visible',
					//'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Category',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_CATEGORY_CREATE => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'1' => 'Yes',
					'0' => 'No',
					//'required' => 'Required',
				),
				'default' => '0',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Create Category',
				'desc' => 'If enabled, then user able to create new categories from frontend form.
				<br>Notice: This option will show if above category field set visible.',
			),
			Settings::OPTION_LOCATION_FORM_MARKERICON => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Visible',
					//'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Marker icon',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_MARKERICON_UPLOAD => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Visible',
					//'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Upload marker icon',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_TAGS => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Optional',
					'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Tags',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_IMAGES => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Visible',
					//'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Images',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_VIDEOS => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Visible',
					//'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Videos',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_LATITUDE => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Visible',
					//'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Latitude',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_LONGITUDE => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Visible',
					//'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Longitude',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_MARKERALT => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Optional',
					'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Marker Alt',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_ADDRESS => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Optional',
					'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Address',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_POSTALCODE => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Optional',
					'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Postal code',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_PHONENUMBER => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Optional',
					'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Phone Number',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_WEBSITE => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Optional',
					'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Website',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_EMAILADDRESS => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Optional',
					'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Email address',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_URL => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Optional',
					'required' => 'Required',
				),
				'default' => 'optional',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'URL',
				'desc' => '',
			),
			Settings::OPTION_LOCATION_FORM_TERMS => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Visible',
				),
				'default' => 'none',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Terms & Conditions',
				'desc' => 'If visible, then terms & conditions checkbox will be show in add location form and without accepting user can not be submit',
			),
			Settings::OPTION_LOCATION_FORM_LINKSHARING => array(
				'type' => Settings::TYPE_RADIO,
				'options' => array(
					'none' => 'None',
					'optional' => 'Visible',
				),
				'default' => 'none',
				'category' => 'locationform',
				'subcategory' => 'locationform',
				'title' => 'Link Sharing',
				'desc' => '',
			),

			// Access Control
			Settings::OPTION_ACCESS_MAP_INDEX => array(
				'type' => Settings::TYPE_SELECT,
				'options' => array(App::namespaced('model\Settings'), 'getAccessOptions'),
				'default' => Settings::ACCESS_GUEST,
				'category' => 'access',
				'subcategory' => 'access',
				'title' => 'List locations',
				'desc' => 'Select who can access the locations index and also search or filter locations.',
			),
			Settings::OPTION_ACCESS_MAP_INDEX_CAP => array(
				'type' => Settings::TYPE_STRING,
				'category' => 'access',
				'subcategory' => 'access',
				'default' => '',
				'title' => 'List locations custom capability',
				'desc' => 'Enter a capability name which will be required for user to show locations index, search or filter locations.<br />'
				. 'Read about <a href="https://codex.wordpress.org/Roles_and_Capabilities">Roles and Capabilities</a> on Wordpress Codex.',
			),
			Settings::OPTION_ACCESS_MAP_VIEW => array(
				'type' => Settings::TYPE_SELECT,
				'options' => array(App::namespaced('model\Settings'), 'getAccessOptions'),
				'default' => Settings::ACCESS_GUEST,
				'category' => 'access',
				'subcategory' => 'access',
				'title' => 'View location',
				'desc' => 'Select who can display the location\'s page.',
			),
			Settings::OPTION_ACCESS_MAP_VIEW_CAP => array(
				'type' => Settings::TYPE_STRING,
				'category' => 'access',
				'subcategory' => 'access',
				'default' => '',
				'title' => 'View map custom capability',
				'desc' => 'Enter a capability name which will be required for user to display a map page.<br />'
					. 'Read about <a href="https://codex.wordpress.org/Roles_and_Capabilities">Roles and Capabilities</a> on Wordpress Codex.',
			),
			Settings::OPTION_ACCESS_MAP_CREATE => array(
				'type' => Settings::TYPE_SELECT,
				'options' => array(App::namespaced('model\Settings'), 'getAccessOptionsWithoutGuest'),
				'default' => Settings::ACCESS_USER,
				'category' => 'access',
				'subcategory' => 'access',
				'title' => 'Create location',
				'desc' => 'Select who can create locations.',
			),
			Settings::OPTION_ACCESS_MAP_CREATE_CAP => array(
				'type' => Settings::TYPE_STRING,
				'category' => 'access',
				'subcategory' => 'access',
				'default' => 'read',
				'title' => 'Create location capability',
				'desc' => 'Enter a capability name which will be required for user to create or update own location.<br />'
					. 'Read about <a href="https://codex.wordpress.org/Roles_and_Capabilities">Roles and Capabilities</a> on Wordpress Codex.',
			),
			Settings::OPTION_ACCESS_MAP_EDIT => array(
				'type' => Settings::TYPE_SELECT,
				'options' => array(App::namespaced('model\Settings'), 'getAccessOptionsWithoutGuest'),
				'default' => Settings::ACCESS_USER,
				'category' => 'access',
				'subcategory' => 'access',
				'title' => 'Update own location',
				'desc' => 'Select who can update own locations.',
			),
			Settings::OPTION_ACCESS_MAP_EDIT_CAP => array(
				'type' => Settings::TYPE_STRING,
				'category' => 'access',
				'subcategory' => 'access',
				'default' => 'read',
				'title' => 'Update own location capability',
				'desc' => 'Enter a capability name which will be required for user to update own location.<br />'
				. 'Read about <a href="https://codex.wordpress.org/Roles_and_Capabilities">Roles and Capabilities</a> on Wordpress Codex.',
			),
			Settings::OPTION_ACCESS_MEDIA_LIBRARY_ROLES => array(
				'type' => Settings::TYPE_MULTICHECKBOX,
				'options' => function() { return Settings::getRolesOptions(); },
				'default' => function() { return array_keys(Settings::getRolesOptions()); },
				'category' => 'access',
				'subcategory' => 'access',
				'title' => 'Roles allowed to use Media Library',
				'desc' => 'If a subscriber has the upload_files capability he can see the Wordpress\'s Media Library tab when uploading the image for a location. '
						. 'If you want to prevent for users browsing your website\'s Media Library revoke access for the specific roles.',
			),
			
			Settings::OPTION_INDEX_MAP_MARKER_CLICK => array(
				'type' => Settings::TYPE_RADIO,
				'default' => Settings::DEFAULT_INDEX_MAP_MARKER_CLICK,
				'options' => array(
					Settings::ACTION_CLICK_NONE => 'None',
					Settings::ACTION_CLICK_REDIRECT => 'Open the location\'s page',
					Settings::ACTION_CLICK_CUSTOM_REDIRECT => 'Redirect to custom URL',
					Settings::ACTION_CLICK_TOOLTIP => 'Show tooltip with information about the location',
				),
				'category' => 'index',
				'subcategory' => 'map',
				'title' => 'Clicking on the map marker will',
			),
			Settings::OPTION_INDEX_LIST_ITEM_CLICK => array(
				'type' => Settings::TYPE_RADIO,
				'default' => Settings::DEFAULT_INDEX_LIST_ITEM_CLICK,
				'options' => array(
					Settings::ACTION_CLICK_NONE => 'None',
					Settings::ACTION_CLICK_REDIRECT => 'Open the location\'s page',
					Settings::ACTION_CLICK_CUSTOM_REDIRECT => 'Redirect to custom URL',
					Settings::ACTION_CLICK_TOOLTIP => 'Show tooltip with information about the location',
				),
				'category' => 'index',
				'subcategory' => 'appearance',
				'title' => 'Clicking on the location\'s list item will',
			),
			Settings::OPTION_TOOLTIP_DESCRIPTION_CHARS => array(
				'type' => Settings::TYPE_INT,
				'default' => Settings::DEFAULT_TOOLTIP_DESCRIPTION_CHARS,
				'category' => 'index',
				'subcategory' => 'infowindow',
				'title' => 'Number of characters from the location\'s description to display on the tooltip',
			),
			
			Settings::OPTION_INDEX_ZIP_RADIUS_FILTER_ENABLE => array(
				'type' => Settings::TYPE_BOOL,
				'default' => 0,
				'category' => 'index',
				'subcategory' => 'zip',
				'title' => 'Enable ZIP code neighborhood filter',
				'desc' => 'If enabled the ZIP code neighborhood filter will be added to the index page next to the search box.',
			),
			Settings::OPTION_INDEX_ZIP_RADIUS_FILTER_POSITION => array(
				'type' => Settings::TYPE_BOOL,
				'default' => 0,
				'category' => 'index',
				'subcategory' => 'zip',
				'title' => 'Position (after category filter)',
				'desc' => 'If enabled then filter will be added next to the category filter otherwise next to search box.',
			),
			Settings::OPTION_INDEX_ZIP_RADIUS_COUNTRY => array(
				'type' => Settings::TYPE_STRING,
				'default' => 'USA',
				'category' => 'index',
				'subcategory' => 'zip',
				'title' => 'Country code for the ZIP code searching',
				'desc' => 'The ZIP filter will work only within a single country.',
			),
			Settings::OPTION_INDEX_ZIP_RADIUS_MIN => array(
				'type' => Settings::TYPE_INT,
				'default' => 10,
				'category' => 'index',
				'subcategory' => 'zip',
				'title' => 'Minimum radius value',
				'desc' => 'Set the minimum radius value that user will be able to select from the drop-box field placed next to the zip code filter.',
			),
			Settings::OPTION_INDEX_ZIP_RADIUS_MAX => array(
				'type' => Settings::TYPE_INT,
				'default' => 1000,
				'category' => 'index',
				'subcategory' => 'zip',
				'title' => 'Maximum radius value',
				'desc' => 'Set the maximum radius value that user will be able to select from the drop-box field placed next to the zip code filter.',
			),
			Settings::OPTION_INDEX_ZIP_RADIUS_STEP => array(
				'type' => Settings::TYPE_INT,
				'default' => 10,
				'category' => 'index',
				'subcategory' => 'zip',
				'title' => 'Radius value step',
				'desc' => 'Set the radius step that will be used to generated radius values from minimum to maximum.',
			),
			Settings::OPTION_INDEX_ZIP_RADIUS_DEFAULT => array(
				'type' => Settings::TYPE_INT,
				'default' => 10,
				'category' => 'index',
				'subcategory' => 'zip',
				'title' => 'Radius default value',
				'desc' => 'Set the default radius value that will be selected in the drop-box field placed next to the zip code filter.',
			),
			Settings::OPTION_INDEX_ZIP_RADIUS_GEOLOCATION => array(
				'type' => Settings::TYPE_BOOL,
				'default' => 0,
				'category' => 'index',
				'subcategory' => 'zip',
				'title' => 'Enable geolocation',
				'desc' => 'If enabled the user\'s ZIP code will be recognized using browser\'s geolocation API.'
						.'<br />Notice that the geolocation API works only if you\'re using https.',
			),
			
			Settings::OPTION_MAP_DEFAULT_ICON_URL => array(
				'type' => Settings::TYPE_STRING,
				'default' => '',
				'category' => 'index',
				'subcategory' => 'map',
				'title' => 'Default location\'s icon URL',
				'desc' => 'If there\'s not location\'s icon this will be used instead.',
			),
			Settings::OPTION_SINGLE_ROUTE_RATING_SHOW => array(
				'type' => Settings::TYPE_BOOL,
				'default' => true,
				'category' => 'location',
				'subcategory' => 'appearance',
				'title' => 'Show rating on the location\'s page',
			),
			Settings::OPTION_INDEX_RATING_SHOW => array(
				'type' => Settings::TYPE_BOOL,
				'default' => false,
				'category' => 'index',
				'subcategory' => 'appearance',
				'title' => 'Show rating on the index page',
			),
			
			Settings::OPTION_SINGLE_ROUTE_EMBED_ENABLE => array(
				'type' => Settings::TYPE_BOOL,
				'default' => true,
				'category' => 'location',
				'subcategory' => 'appearance',
				'title' => 'Show "Embed" feature',
				'desc' => 'If enabled, the "Embed" button will be display next to the location\'s map and users will be able to copy the iframe HTML code '
				. 'in order to embed the map on an external website.'
			),
			
		));
	}
	
	static function cmloc_load_assets_frontend() {
		wp_enqueue_script('cmloc-froogaloop', App::url('asset/js/froogaloop/froogaloop-min.js'), null, App::VERSION);
	}
	
	static function cmloc_route_single_params_names($params) {
		$params['weather'] = 'Weather';
		return $params;
	}
	
	static function cmloc_route_index_params_names($params) {
		//$params['featured_image'] = 'Featured image';
		return $params;
	}
	
	static function cmloc_single_location_before_images(Location $location) {
		printf('<a target="_blank" class="cmloc-weather" title="%s"></a>', esc_attr(Labels::getLocalized('check_weather')));
	}
	
	static function cmloc_settings_pages($categories) {
		$categories['access'] = 'Access Control';
		$categories['locationform'] = 'Location Form';
		$categories['labels'] = 'Labels';
		return $categories;
	}
	
	static function cmloc_route_editor_middle(Route $route) {
		$iconSizes = array(Route::ICON_SIZE_SMALL, Route::ICON_SIZE_NORMAL, Route::ICON_SIZE_LARGE);
		$iconSizes = array_combine($iconSizes, array_map(function($key) { return Labels::getLocalized('icon_size_' . $key); }, $iconSizes));
		echo DashboardController::loadFrontendView('editor-icon', compact('route', 'iconSizes'));
	}
	
	static function cmloc_route_icon_url($iconUrl, $route) {
		if ($defaultIcon = Settings::getOption(Settings::OPTION_MAP_DEFAULT_ICON_URL) AND empty($iconUrl)) {
			$iconUrl = $defaultIcon;
		}
		return $iconUrl;
	}

}