<?php

namespace com\cminds\maplocations\controller;

use com\cminds\maplocations\model\Attachment;
use com\cminds\maplocations\shortcode\MapShortcode;
use com\cminds\maplocations\shortcode\LocationSnippetShortcode;
use com\cminds\maplocations\shortcode\SearchShortcode;
use com\cminds\maplocations\model\Location;
use com\cminds\maplocations\model\Labels;
use com\cminds\maplocations\App;
use com\cminds\maplocations\model\Route;
use com\cminds\maplocations\model\Settings;
use com\cminds\maplocations\model\Category;

class RouteController extends Controller {

	const PARAM_PAGE = 'page';
	const UPLOAD_ICON_NONCE = 'cmloc_upload_icon_nonce';
	const ICON_MAX_DIMENSION = 100;

	static $filters = array(
		'cmloc_route_index_single' => array( 'args' => 2 ),
		'posts_search'             => array( 'args' => 2 ),
	);
	static $actions = array(
		array( 'name' => 'pre_get_posts', 'args' => 1, 'priority' => PHP_INT_MAX ),
		array( 'name' => 'get_template_part_cmloc', 'args' => 2 ),
		'cmloc_route_index_pagination'  => array( 'args' => 1, 'method' => 'displayPagination' ),
		'cmloc_route_index_filter',
		'cmloc_route_single_before',
		'cmloc_route_single_map',
		'cmloc_route_single_details',
		'cmloc_route_single_locations',
		'cmloc_single_location_address' => array( 'args' => 1 ),
		'wp_enqueue_scripts',
	);
	static $ajax = array(
		'cmloc_map_shortcode',
		'cmloc_upload_icon',
		'cmloc_get_route_infowindow',
	);

	static $mapId = null;


	static function indexView( \WP_Query $query ) {
		if ( Route::canViewIndex() ) {
			$routes            = array_map( array( App::namespaced( 'model\Route' ), 'getInstance' ), $query->posts );
			$totalRoutesNumber = FrontendController::$query->found_posts;
// 			echo FrontendController::$query->request;exit;
			$displayParams = Settings::getOption( Settings::OPTION_INDEX_ROUTE_PARAMS );
			$layout        = Settings::getOption( Settings::OPTION_INDEX_LOCATIONS_LIST_LAYOUT );
			if ( empty( $layout ) ) {
				$layout = Settings::INDEX_LIST_BOTTOM;
			}
			wp_enqueue_script( 'cmloc-index-geolocation' );

			return self::loadFrontendView( 'index', compact( 'routes', 'totalRoutesNumber', 'displayParams', 'layout' ) );
		} else {
			return Labels::getLocalized( 'location_index_access_denied' );
		}
	}


	static function singleView( \WP_Query $query ) {
		global $id, $post, $withcomments;

		if ( ! empty( $query->posts[0] ) AND $route = Route::getInstance( $query->posts[0] ) ) {
			if ( $route->canView() ) {

				$post  = $query->posts[0];
				$mapId = static::$mapId = 'cmloc-route-' . mt_rand();

				$id = $route->getId();
				$route->incrementViews();
				$withcomments  = true;
				$displayParams = Settings::getOption( Settings::OPTION_SINGLE_ROUTE_PARAMS );
				$mapStyle      = self::getStyle();


				return self::loadFrontendView( 'single', compact( 'route', 'mapId', 'displayParams', 'mapStyle' ) );

			} else {
				return Labels::getLocalized( 'location_access_denied' );
			}

		} else {
			return Labels::getLocalized( 'location_not_found' );
		}
	}

	static function getStyle( $setting = false ) {
		if ( ! $setting ) {
			$setting = Settings::getOption( 'cmloc_map_themes' );
		}
		switch ( $setting ) {
			case "silver":
				return "[{\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#f5f5f5\"}]},{\"elementType\":\"labels.icon\",\"stylers\":[{\"visibility\":\"off\"}]},{\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#616161\"}]},{\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#f5f5f5\"}]},{\"featureType\":\"administrative.land_parcel\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#bdbdbd\"}]},{\"featureType\":\"poi\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#eeeeee\"}]},{\"featureType\":\"poi\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#757575\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#e5e5e5\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#9e9e9e\"}]},{\"featureType\":\"road\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#ffffff\"}]},{\"featureType\":\"road.arterial\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#757575\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#dadada\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#616161\"}]},{\"featureType\":\"road.local\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#9e9e9e\"}]},{\"featureType\":\"transit.line\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#e5e5e5\"}]},{\"featureType\":\"transit.station\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#eeeeee\"}]},{\"featureType\":\"water\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#c9c9c9\"}]},{\"featureType\":\"water\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#9e9e9e\"}]}]";
				break;
			case "retro":
				return "[{\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#ebe3cd\"}]},{\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#523735\"}]},{\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#f5f1e6\"}]},{\"featureType\":\"administrative\",\"elementType\":\"geometry.stroke\",\"stylers\":[{\"color\":\"#c9b2a6\"}]},{\"featureType\":\"administrative.land_parcel\",\"elementType\":\"geometry.stroke\",\"stylers\":[{\"color\":\"#dcd2be\"}]},{\"featureType\":\"administrative.land_parcel\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#ae9e90\"}]},{\"featureType\":\"landscape.natural\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#dfd2ae\"}]},{\"featureType\":\"poi\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#dfd2ae\"}]},{\"featureType\":\"poi\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#93817c\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"geometry.fill\",\"stylers\":[{\"color\":\"#a5b076\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#447530\"}]},{\"featureType\":\"road\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#f5f1e6\"}]},{\"featureType\":\"road.arterial\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#fdfcf8\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#f8c967\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"geometry.stroke\",\"stylers\":[{\"color\":\"#e9bc62\"}]},{\"featureType\":\"road.highway.controlled_access\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#e98d58\"}]},{\"featureType\":\"road.highway.controlled_access\",\"elementType\":\"geometry.stroke\",\"stylers\":[{\"color\":\"#db8555\"}]},{\"featureType\":\"road.local\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#806b63\"}]},{\"featureType\":\"transit.line\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#dfd2ae\"}]},{\"featureType\":\"transit.line\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#8f7d77\"}]},{\"featureType\":\"transit.line\",\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#ebe3cd\"}]},{\"featureType\":\"transit.station\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#dfd2ae\"}]},{\"featureType\":\"water\",\"elementType\":\"geometry.fill\",\"stylers\":[{\"color\":\"#b9d3c2\"}]},{\"featureType\":\"water\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#92998d\"}]}]";
				break;
			case "dark":
				return "[{\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#212121\"}]},{\"elementType\":\"labels.icon\",\"stylers\":[{\"visibility\":\"off\"}]},{\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#757575\"}]},{\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#212121\"}]},{\"featureType\":\"administrative\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#757575\"}]},{\"featureType\":\"administrative.country\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#9e9e9e\"}]},{\"featureType\":\"administrative.land_parcel\",\"stylers\":[{\"visibility\":\"off\"}]},{\"featureType\":\"administrative.locality\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#bdbdbd\"}]},{\"featureType\":\"poi\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#757575\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#181818\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#616161\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#1b1b1b\"}]},{\"featureType\":\"road\",\"elementType\":\"geometry.fill\",\"stylers\":[{\"color\":\"#2c2c2c\"}]},{\"featureType\":\"road\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#8a8a8a\"}]},{\"featureType\":\"road.arterial\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#373737\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#3c3c3c\"}]},{\"featureType\":\"road.highway.controlled_access\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#4e4e4e\"}]},{\"featureType\":\"road.local\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#616161\"}]},{\"featureType\":\"transit\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#757575\"}]},{\"featureType\":\"water\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#000000\"}]},{\"featureType\":\"water\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#3d3d3d\"}]}]";
				break;
			case "night":
				return "[{\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#242f3e\"}]},{\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#746855\"}]},{\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#242f3e\"}]},{\"featureType\":\"administrative.locality\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#d59563\"}]},{\"featureType\":\"poi\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#d59563\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#263c3f\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#6b9a76\"}]},{\"featureType\":\"road\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#38414e\"}]},{\"featureType\":\"road\",\"elementType\":\"geometry.stroke\",\"stylers\":[{\"color\":\"#212a37\"}]},{\"featureType\":\"road\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#9ca5b3\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#746855\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"geometry.stroke\",\"stylers\":[{\"color\":\"#1f2835\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#f3d19c\"}]},{\"featureType\":\"transit\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#2f3948\"}]},{\"featureType\":\"transit.station\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#d59563\"}]},{\"featureType\":\"water\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#17263c\"}]},{\"featureType\":\"water\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#515c6d\"}]},{\"featureType\":\"water\",\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#17263c\"}]}]";
				break;
			case "aubergine":
				return "[{\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#1d2c4d\"}]},{\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#8ec3b9\"}]},{\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#1a3646\"}]},{\"featureType\":\"administrative.country\",\"elementType\":\"geometry.stroke\",\"stylers\":[{\"color\":\"#4b6878\"}]},{\"featureType\":\"administrative.land_parcel\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#64779e\"}]},{\"featureType\":\"administrative.province\",\"elementType\":\"geometry.stroke\",\"stylers\":[{\"color\":\"#4b6878\"}]},{\"featureType\":\"landscape.man_made\",\"elementType\":\"geometry.stroke\",\"stylers\":[{\"color\":\"#334e87\"}]},{\"featureType\":\"landscape.natural\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#023e58\"}]},{\"featureType\":\"poi\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#283d6a\"}]},{\"featureType\":\"poi\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#6f9ba5\"}]},{\"featureType\":\"poi\",\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#1d2c4d\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"geometry.fill\",\"stylers\":[{\"color\":\"#023e58\"}]},{\"featureType\":\"poi.park\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#3C7680\"}]},{\"featureType\":\"road\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#304a7d\"}]},{\"featureType\":\"road\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#98a5be\"}]},{\"featureType\":\"road\",\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#1d2c4d\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#2c6675\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"geometry.stroke\",\"stylers\":[{\"color\":\"#255763\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#b0d5ce\"}]},{\"featureType\":\"road.highway\",\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#023e58\"}]},{\"featureType\":\"transit\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#98a5be\"}]},{\"featureType\":\"transit\",\"elementType\":\"labels.text.stroke\",\"stylers\":[{\"color\":\"#1d2c4d\"}]},{\"featureType\":\"transit.line\",\"elementType\":\"geometry.fill\",\"stylers\":[{\"color\":\"#283d6a\"}]},{\"featureType\":\"transit.station\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#3a4762\"}]},{\"featureType\":\"water\",\"elementType\":\"geometry\",\"stylers\":[{\"color\":\"#0e1626\"}]},{\"featureType\":\"water\",\"elementType\":\"labels.text.fill\",\"stylers\":[{\"color\":\"#4e6d70\"}]}]";
				break;
			default:
				return "[]";
		}
	}


	static function wp_enqueue_scripts() {
		if ( FrontendController::isRoutePostType() ) {
			FrontendController::enqueueStyle();
		}
		if ( FrontendController::isRouteSinglePage() ) {
			self::loadSinglePageScripts();
		}
	}


	static function loadSinglePageScripts() {
		wp_enqueue_script( 'cmloc-location-map' );
		wp_enqueue_script( 'cmloc-frontend' );
		do_action( 'cmloc_load_single_page_scripts' );
	}


	static function get_template_part_cmloc( $slug, $name ) {
		switch ( $name ) {
			case 'route-index-filter':
				wp_enqueue_script( 'cmloc-index-filter' );
				self::displayIndexTop();
				do_action( 'cmloc_route_index_filter' );
				break;
			case 'route-index-map':
				self::displayIndexMap();
				break;
			case 'route-index-list':
				echo self::getIndexList();
				break;
			case 'route-single-before':
				do_action( 'cmloc_route_single_before' );
				break;
			case 'route-single-map':
				do_action( 'cmloc_route_single_map' );
				break;
			case 'route-single-details':
				do_action( 'cmloc_route_single_details' );
				break;
			case 'route-single-locations':
				do_action( 'cmloc_route_single_locations' );
				break;
		}
	}


	static function displayIndexTop() {
		$text = trim( Settings::getOption( Settings::OPTION_INDEX_TEXT_TOP ) );
		echo self::loadFrontendView( 'index-top', compact( 'text' ) );
	}


	static function displayIndexMap() {
		$routes = Route::getIndexMapJSLocations( FrontendController::$query );
// 		if (!empty($routes)) {
		$mapStyle = self::getStyle();
		wp_enqueue_script( 'cmloc-index-map' );
		echo self::loadFrontendView( 'index-map', compact( 'routes','mapStyle' ) );
// 		}
	}


	static function getIndexList( \WP_Query $query = null ) {
		if ( empty( $query ) ) {
			$query = FrontendController::$query;
		}
		$routes            = array_map( array( App::namespaced( 'model\Route' ), 'getInstance' ), $query->posts );
		$totalRoutesNumber = $query->found_posts;
		$displayParams     = Settings::getOption( Settings::OPTION_INDEX_ROUTE_PARAMS );

		return self::loadFrontendView( 'index-list', compact( 'routes', 'totalRoutesNumber', 'displayParams', 'query' ) );
	}


	static function displayPagination( $query = null ) {
		if ( empty( $query ) ) {
			$query = FrontendController::$query;
		}
		$limit = Route::getPaginationLimit();
		if ( $query->found_posts > $limit ) {
			$total_pages = $query->max_num_pages;
			$page        = $query->get( 'paged' );
			if ( empty( $page ) ) {
				$page = 1;
			}
			$base_url = static::getPaginationBaseUrl();
			echo self::loadView( 'frontend/common/pagination', compact( 'total_pages', 'page', 'base_url' ) );
		}
	}


	static function getPaginationBaseUrl() {
		return preg_replace( '~/page/[0-9]+/~', '/', $_SERVER['REQUEST_URI'] );
	}


	static function cmloc_route_index_filter() {
		if ( $category = FrontendController::getCategory() ) {
			$searchFormUrl = $category->getPermalink();
		} else {
			$searchFormUrl = FrontendController::getUrl();
		}
		if ( App::isPro() ) {
			echo SearchShortcode::shortcode( array(
				'searchformurl' => $searchFormUrl,
				'searchstring'  => filter_input( INPUT_GET, 's' ),
			) );
// 			echo self::loadFrontendView('index-filter', compact('searchFormUrl'));
		}
	}


	static function cmloc_route_single_before() {
		$route = FrontendController::getRoute();
		echo self::loadFrontendView( 'single-before', compact( 'route' ) );
	}

	static function cmloc_route_single_map() {
		$route = FrontendController::getRoute();
		$mapId = static::$mapId;
		$atts  = array();
		static::displaySingleMap( $route, $atts, $mapId );
// 		$mapId = static::$mapId;
// 		$route = FrontendController::getRoute();
// 		$mapCanvas = static::getMapCanvas($route, $mapId);
// 		echo self::loadFrontendView('single-map', compact('route', 'mapId', 'mapCanvas'));
	}


	static function displaySingleMap( Route $route, $atts = array(), $mapId = null ) {
		$mapCanvas = static::getMapCanvas( $route, $mapId );
		echo self::loadFrontendView( 'single-map', compact( 'route', 'mapId', 'mapCanvas' ) );
	}


	static function getMapCanvas( $route, $mapId ) {
		return static::loadFrontendView( 'single-map-canvas', compact( 'route', 'mapId' ) );
	}

	static function cmloc_route_single_details() {
		$route = FrontendController::getRoute();
		echo self::loadFrontendView( 'single-details', compact( 'route' ) );
	}

	static function cmloc_route_single_locations() {
		$route = FrontendController::getRoute();
		echo self::loadFrontendView( 'single-locations', compact( 'route' ) );
	}


	static function cmloc_route_index_single( $output, $route ) {
		return self::loadFrontendView( 'index-single', compact( 'route' ) );
	}


	static function getDashboardUrl( $action = 'index', $params = array() ) {
		return FrontendController::getUrl( FrontendController::URL_DASHBOARD . '/' . $action, $params );
	}


	static function pre_get_posts( \WP_Query $query ) {
		if ( is_admin() ) {
			return;
		}
		if ( $query->is_main_query() AND FrontendController::isRoutePostType( $query ) ) {
			if ( ! $query->is_single() ) {
				$query->set( 'posts_per_page', Route::getPaginationLimit() );
				Route::registerQueryOrder( $query );
			}
			if ( ! FrontendController::isDashboard( $query ) ) {
				$query->set( 'post_status', 'publish' );
			}
		}
		if ( $query->is_main_query() AND $categorySlug = $query->get( Category::TAXONOMY ) ) {
			$query->set( 'post_type', Route::POST_TYPE );
		}
	}


	static function posts_search( $search, \WP_Query $query ) {
		global $wpdb;
		$str = $query->get( 's' );
		if ( $str == '' ) {
			$str = ( isset( $_GET['s'] ) ) ? $_GET['s'] : '';
		}
		if ( strlen( $str ) > 0 AND $query->is_main_query() AND FrontendController::isRoutePostType( $query ) ) {
			// Additional search by address and postal code
			$str    = '%' . $str . '%';
			$sql    = $wpdb->prepare( ') OR (cmloc_addr.meta_value LIKE %s OR cmloc_postcode.meta_value LIKE %s OR cmloc_lat.meta_value LIKE %s OR cmloc_long.meta_value LIKE %s OR cmloc_lat_long.meta_value LIKE %s) OR (', $str, $str, $str, $str, $str );
			$search = str_replace( ') OR (', $sql, $search );
			// Add required joins
			add_filter( 'posts_join', array( __CLASS__, 'posts_search_join' ), 10, 2 );
		}

		return $search;
	}


	static function posts_search_join( $join, \WP_Query $query ) {
		global $wpdb;
		// Additional joins to search by address and postal code
		$join .= PHP_EOL . "JOIN $wpdb->posts cmloc_location ON cmloc_location.post_parent = $wpdb->posts.ID";
		$join .= PHP_EOL . $wpdb->prepare( "JOIN $wpdb->postmeta cmloc_addr ON cmloc_addr.post_id = cmloc_location.ID AND cmloc_addr.meta_key = %s", Location::META_ADDRESS );
		$join .= PHP_EOL . $wpdb->prepare( "JOIN $wpdb->postmeta cmloc_postcode ON cmloc_postcode.post_id = cmloc_location.ID AND cmloc_postcode.meta_key = %s", Location::META_POSTAL_CODE );
		$join .= PHP_EOL . $wpdb->prepare( "JOIN $wpdb->postmeta cmloc_lat ON cmloc_lat.post_id = cmloc_location.ID AND cmloc_lat.meta_key = %s", Location::META_LAT );
		$join .= PHP_EOL . $wpdb->prepare( "JOIN $wpdb->postmeta cmloc_long ON cmloc_long.post_id = cmloc_location.ID AND cmloc_long.meta_key = %s", Location::META_LONG );
		$join .= PHP_EOL . $wpdb->prepare( "JOIN $wpdb->postmeta cmloc_lat_long ON cmloc_lat_long.post_id = cmloc_location.ID AND cmloc_lat_long.meta_key = %s", Location::META_LAT_LONG );
		$join .= PHP_EOL;
		remove_filter( 'posts_join', array( __CLASS__, 'posts_search_join' ), 10 );

		return $join;
	}


	static function getInfoWindowView( Route $route ) {
		$snippet = LocationSnippetShortcode::shortcode( array( 'route' => $route ) );

		if ( $maxlen = Settings::getTooltipDescriptionCharsNumber() ) {
			$description = $route->getShortDescription( $maxlen );
		} else {
			$description = $route->getShortDescription();
		}

		$location_info_window_template = Settings::getOption( Settings::OPTION_MAP_LOCATION_INFO_WINDOW_TEMPLATE );
		if ( $location_info_window_template != '' ) {
			$content  = $location_info_window_template;
			$loc_data = get_post( $route->getID() );
			$content  = str_replace( '[title]', $loc_data->post_title, $content );
			$content  = str_replace( '[description]', $description, $content );
			$content  = str_replace( '[address]', $route->getLocation()->getAddress(), $content );
			$content  = str_replace( '[latitude]', $route->getLocation()->getLat(), $content );
			$content  = str_replace( '[longitude]', $route->getLocation()->getLong(), $content );
			$content  = str_replace( '[postalcode]', $route->getLocation()->getPostalCode(), $content );
			$content  = str_replace( '[phone]', $route->getLocation()->getPhoneNumber(), $content );
			$content  = str_replace( '[website]', $route->getLocation()->getWebsite(), $content );
			$content  = str_replace( '[email]', $route->getLocation()->getEmail(), $content );
			$content  = str_replace( '[permalink]', $route->getPermalink(), $content );
			$content  = str_replace( '[imagesrc]', $route->getLocation()->getFirstImageSrc(), $content );
			$content  = str_replace( '[image]', $route->getLocation()->getFirstImage(), $content );
			$editlink = '';
			if ( is_user_logged_in() && current_user_can( 'administrator' ) ) {
				$editlink = '<a href="' . get_edit_post_link( $loc_data->ID ) . '" target="_blank">' . Labels::getLocalized( 'dashboard_edit' ) . '</a>';
			}
			$content = str_replace( '[editlink]', $editlink, $content );

			return $content;
		} else {
			return self::loadFrontendView( 'infowindow', compact( 'snippet', 'description', 'route' ) );
		}

	}


	static function cmloc_map_shortcode() {
		if ( ! empty( $_POST['params'] ) ) {
			$params = $_POST['params'];
			echo MapShortcode::shortcode( $params );
		}
	}


	static function cmloc_upload_icon() {
		$response = array( 'success' => false, 'msg' => 'An error occurred.' );
		if ( ! empty( $_FILES['cmloc-upload-icon'] ) AND ! empty( $_POST['nonce'] ) AND wp_verify_nonce( $_POST['nonce'], self::UPLOAD_ICON_NONCE ) ) {
			try {
				$targetDirectory = Attachment::getUploadDir( Attachment::UPLOAD_DIR_LOCATION_ICONS );
				if ( ! Attachment::validateExtension( $_FILES['cmloc-upload-icon']['name'], Attachment::$imageExtensions ) ) {
					throw new \Exception( 'File must be an image.' );
				}
				$filePath = Attachment::upload( $_FILES['cmloc-upload-icon'], $targetDirectory );
				Attachment::imageResizeCrop( $filePath, self::ICON_MAX_DIMENSION, self::ICON_MAX_DIMENSION );
				$url      = Attachment::getUrlByPath( $filePath );
				$response = array( 'success' => true, 'msg' => 'OK', 'iconUrl' => $url );
			} catch ( \Exception $e ) {
				$response['error'] = $e->getMessage();
			}
		}
		header( 'content-type: application/json' );
		echo json_encode( $response );
		exit;
	}


	static function cmloc_single_location_address( Location $location ) {
		echo static::loadFrontendView( 'location-address', compact( 'location' ) );
	}


	static function cmloc_get_route_infowindow() {
		if ( $id = filter_input( INPUT_POST, 'id' ) AND $route = Route::getInstance( $id ) ) {
			echo static::getInfoWindowView( $route );
		}
		exit;
	}


}
