<?php
namespace com\cminds\maplocations\shortcode;

use com\cminds\maplocations\model\Category;
use com\cminds\maplocations\model\MapRoutesCategory;
use com\cminds\maplocations\helper\RouteView;
use com\cminds\maplocations\model\MapRoute;
use com\cminds\maplocations\controller\FrontendController;
use com\cminds\maplocations\model\Settings;
use com\cminds\maplocations\controller\RouteController;
use com\cminds\maplocations\model\Route;

class MapRoutesCommonShortcode extends Shortcode {
	
	const SHORTCODE_NAME = 'cmmrm-cmloc-common-map';
	const CACHE_PREFIX = '_cmloccache_common_shcode_';
	const CACHE_TIME = 0;
	
	static function shortcode($atts) {
		
		//$start = microtime(true);
		
		$atts = shortcode_atts(array(
			'showonlybyparams' => 0,
			'path' => 0,
			'categoryfilter' => 0,
			'categoryfilterautoselect' => '',
			'cache' => 0,
			'theme' => false,
			'mapwidth' => '',
			'mapheight' => '',
			'width' => '',
			'list' => 'none',
			'menu' => '0',
		), $atts);
		
		// Disabled the caching feature since the script has been moved to wp_footer
		$atts['cache'] = 0;
		
		// Load resources
		FrontendController::enqueueStyle();
		wp_enqueue_script('cmloc-index-map');
		RouteController::loadSinglePageScripts();


		$mapStyle = RouteController::getStyle($atts['theme']);

		if ($atts['categoryfilter'] || (isset($atts['showonlybyparams']) AND $atts['showonlybyparams'] == '1' AND isset($_GET['category']) && $_GET['category'] != '')) {
			wp_enqueue_script('cmloc-common-map-filter');
		}
		
		// Check cache
		$cacheName = static::CACHE_PREFIX . md5(serialize($atts));
		if ($atts['cache']) {
			$data = get_option($cacheName);
			if (!empty($data)) {
				return $data;
			}
		}
		
		$result = '';
		
		// Query
		$routes = MapRoute::getIndexMapJSLocations();
		//$stop = microtime(true);var_dump(round(($stop-$start)*1000));
		if ($routes) {

			if(isset($atts['showonlybyparams']) AND $atts['showonlybyparams'] == '1') {
				if(isset($_GET['location']) && $_GET['location'] != '') {
					foreach($routes as $routekey=>$routeval) {
						if($routeval['post_name'] != strtolower(urlencode($_GET['location']))) {
							unset($routes[$routekey]);
						}
					}
					$routes = array_values($routes);
				}
			}

			$displayParams = Settings::getOption(Settings::OPTION_INDEX_ROUTE_PARAMS);
			if ($atts['path']) $displayParams[] = 'overview-path';
			
			$content = '';
			if ($atts['categoryfilter'] || (isset($atts['showonlybyparams']) AND $atts['showonlybyparams'] == '1' AND isset($_GET['category']) && $_GET['category'] != '')) {
				$categories = Category::getTreeArray();
				$cmmrm_categories = MapRoutesCategory::getTreeArray();
				$autoselect_categories = '';
				if (!empty($atts['categoryfilterautoselect'])) {
					$autoselect_categories = $atts['categoryfilterautoselect'];
				}
				
				if (isset($atts['showonlybyparams']) AND $atts['showonlybyparams'] == '1') {
					if(isset($_GET['category']) && $_GET['category'] != '') {
						if($autoselect_categories != '') {
							$autoselect_categories .= ',';
						}
						$cmmrm_term_object = get_term_by('slug', $_GET['category'], 'cmmrm_category');
						$autoselect_categories .= $cmmrm_term_object->term_id;
						if($autoselect_categories != '') {
							$autoselect_categories .= ',';
						}
						$cmloc_term_object = get_term_by('slug', $_GET['category'], 'cmloc_category');
						$autoselect_categories .= $cmloc_term_object->term_id;
					}
				}

				$content .= RouteController::loadFrontendView('common-map-category-filter', compact('routes', 'atts', 'categories', 'cmmrm_categories', 'autoselect_categories'));
			}
			$content .= RouteController::loadFrontendView('index-map', compact('mapStyle','routes', 'atts'));
			
			$extra = RouteView::getDisplayParams($displayParams);
			if (!empty($atts['width'])) {
				$extra .= ' style="max-width:'. $atts['width'] .'px;"';
			}
			
			$result = sprintf('<div class="cmloc-map-shortcode cmloc-routes-common-shortcode cmloc-locations-archive"%s>%s</div>',
				$extra,
				$content
			);
			
		}
		
		if ($atts['cache']) {
			update_option($cacheName, $result, false);
		}
		
		//$stop = microtime(true);var_dump(round(($stop-$start)*1000));
		
		if (isset($atts['showonlybyparams']) AND $atts['showonlybyparams'] == '1') {
			if((isset($_GET['location']) && $_GET['location'] != '') || (isset($_GET['category']) && $_GET['category'] != '')) {
				return $result;
			} else {
				return '';
			}
		} else {
			return  $result;
		}
		
	}

}