<?php
namespace com\cminds\maplocations\shortcode;

use com\cminds\maplocations\controller\FrontendController;
use com\cminds\maplocations\helper\RouteView;
use com\cminds\maplocations\model\Category;
use com\cminds\maplocations\model\Settings;
use com\cminds\maplocations\controller\RouteController;
use com\cminds\maplocations\model\Route;

class MapShortcode extends Shortcode {
	
	const SHORTCODE_NAME = 'cmloc-locations-map';
	
	static function shortcode($atts) {
		
		$atts = shortcode_atts(array(
			'showonlybyparams' => 0,
			'location' => null,
			'category' => null,
			'author' => null,
			'limit' => 5,
			'page' => 1,
			'list' => 'none',
			'mapwidth' => '',
			'theme' => false,
			'mapheight' => '',
			'width' => '',
			'menu' => 0,
			'tooltip' => 0,
		), $atts);

		$mapStyle = RouteController::getStyle($atts['theme']);

		if (isset($atts['showonlybyparams']) AND $atts['showonlybyparams'] == '1') {
			if((isset($_GET['location']) && $_GET['location'] != '') || (isset($_GET['category']) && $_GET['category'] != '')) {
				$atts['location'] = $_GET['location'];
				$atts['category'] = $_GET['category'];
			} else {
				return '';
			}
		}

		$cat = null;
		if(!empty($atts['category'])) {
			if(is_numeric($atts['category'])) {
				$term = get_term_by('id', $atts['category'], Category::TAXONOMY);
				if($term){
					$cat = $term->slug;
				}
			} else {
				$cat = $atts['category'];
			}
		}
		
		if($atts['location'] != '') {
			$query = new \WP_Query(array(
				'post_type' => Route::POST_TYPE,
				'post_status' => 'publish',
				Category::TAXONOMY => $cat,
				'posts_per_page' => $atts['limit'],
				'paged' => $atts['page'],
				'author' => $atts['author'],
				'post_name__in' => array($atts['location'])
			));
		} else {
			$query = new \WP_Query(array(
				'post_type' => Route::POST_TYPE,
				'post_status' => 'publish',
				Category::TAXONOMY => $cat,
				'posts_per_page' => $atts['limit'],
				'paged' => $atts['page'],
				'author' => $atts['author']
			));
		}
		$routes = Route::getIndexMapJSLocations($query);
		
		if (!empty($routes)) {
			
			FrontendController::enqueueStyle();
			wp_enqueue_script('cmloc-index-map');
			wp_enqueue_script('cmloc-map-shortcode');
			
			RouteController::loadSinglePageScripts();
			$displayParams = Settings::getOption(Settings::OPTION_INDEX_ROUTE_PARAMS);
			
			$content = RouteController::loadFrontendView('index-map', compact('mapStyle','routes', 'atts'));
			if ($atts['list'] != 'none') $content .= RouteController::getIndexList($query);
			
			$shortcodeId = 'cmloc-map-shortcode-' . mt_rand();
			$content .= sprintf('<script type="text/javascript">jQuery(function($) { CMLOC_Map_Shortcode(%s); });</script>',
				json_encode($shortcodeId)
			);
			
			$extra = RouteView::getDisplayParams($displayParams);
			if (!empty($atts['width'])) {
				$extra .= ' style="max-width:'. $atts['width'] .'px;"';
			}
			
			return sprintf('<div class="cmloc-map-shortcode cmloc-locations-archive cmloc-layout-%s" id="%s" data-use-ajax="1" data-shortcode-params="%s"%s>%s</div>',
				esc_attr($atts['list']),
				esc_attr($shortcodeId),
				esc_attr(json_encode($atts)),
				$extra,
				$content
			);
			
		}

	}

}