<?php
use com\cminds\maplocations\App;
use com\cminds\maplocations\controller;
use com\cminds\maplocations\model\Settings;
use com\cminds\maplocations\model\Labels;
use com\cminds\maplocations\model\Attachment;
use com\cminds\maplocations\model\Route;
use com\cminds\maplocations\helper\FormHtml;
$location_form_description = Settings::getOption(Settings::OPTION_LOCATION_FORM_DESCRIPTION);
$location_form_status = Settings::getOption(Settings::OPTION_LOCATION_FORM_STATUS);
$location_form_images = Settings::getOption(Settings::OPTION_LOCATION_FORM_IMAGES);
$location_form_videos = Settings::getOption(Settings::OPTION_LOCATION_FORM_VIDEOS);
$location_form_latitude = Settings::getOption(Settings::OPTION_LOCATION_FORM_LATITUDE);
$location_form_longitude = Settings::getOption(Settings::OPTION_LOCATION_FORM_LONGITUDE);
$location_form_markeralt = Settings::getOption(Settings::OPTION_LOCATION_FORM_MARKERALT);
$location_form_address = Settings::getOption(Settings::OPTION_LOCATION_FORM_ADDRESS);
$location_form_postalcode = Settings::getOption(Settings::OPTION_LOCATION_FORM_POSTALCODE);
$location_form_phonenumber = Settings::getOption(Settings::OPTION_LOCATION_FORM_PHONENUMBER);
$location_form_website = Settings::getOption(Settings::OPTION_LOCATION_FORM_WEBSITE);
$location_form_emailaddress = Settings::getOption(Settings::OPTION_LOCATION_FORM_EMAILADDRESS);
$location_form_url = Settings::getOption(Settings::OPTION_LOCATION_FORM_URL);
$location_form_terms = Settings::getOption(Settings::OPTION_LOCATION_FORM_TERMS);
?>
<div class="cmloc-location-editor">
    
	<?php do_action('cmloc_route_editor_top', $route); ?>

	<form action="<?php echo esc_attr($formUrl); ?>" method="post" enctype="multipart/form-data" data-route-id="<?php echo $route->getId(); ?>">

		<div class="cmloc-field cmloc-field-location-name">
			<label><?php echo Labels::getLocalized('location_name'); ?>:</label>
			<input type="text" name="name" value="<?php echo esc_attr($route->getTitle()); ?>" required="required" />
		</div>
		
		<?php if ($location_form_description != 'none') { ?>
		<div class="cmloc-field cmloc-field-description">
			<label><?php echo Labels::getLocalized('location_description'); ?>:</label>
			<?php if (Settings::getOption(Settings::OPTION_EDITOR_RICH_TEXT_ENABLE)): ?>
				<?php wp_editor($route->getContent(), 'cmloc_location_description', array('textarea_name' => 'description')); ?>
			<?php else :?>
				<textarea name="description" <?php echo ($location_form_description == 'required')?'required="required"':''; ?>><?php echo esc_html($route->getContent()); ?></textarea>
			<?php endif; ?>
		</div>
		<?php } ?>
		
		<?php if ($location_form_status != 'none') { ?>
		<div class="cmloc-field cmloc-field-location-status">
			<label><?php echo Labels::getLocalized('location_status'); ?>:</label>
			<?php echo FormHtml::selectBox('status', apply_filters('cmloc_editor_allowed_statuses', array(
				'publish' => Labels::getLocalized('location_status_publish'),
				'draft' => Labels::getLocalized('location_status_draft'),
			),  $route), $route->getStatus()); ?>
		</div>
		<?php } else { ?>
			<div class="cmloc-field cmloc-field-location-status" style="display:none;">
				<label><?php echo Labels::getLocalized('location_status'); ?>:</label>
				<?php echo FormHtml::selectBox('status', apply_filters('cmloc_editor_allowed_statuses', array(
					'publish' => Labels::getLocalized('location_status_publish'),
					'draft' => Labels::getLocalized('location_status_draft'),
				),  $route), $route->getStatus()); ?>
			</div>
		<?php } ?>
		
		<?php do_action('cmloc_route_editor_middle', $route); ?>
		
		<?php if ($location_form_images == 'optional' || $location_form_videos == 'optional') { ?>
		<div class="cmloc-field cmloc-images">
			<label><?php echo Labels::getLocalized('location_images'); ?>:</label>
			<ul class="cmloc-images-list">
				<?php
				$template = '<li data-id="%s"%s><a href="%s" target="_blank" title="%s"><img src="%s" alt="Image" /></a>'
					. '<span class="cmloc-image-delete" title="%s">&times;</span></li>';
				printf($template, 0, ' style="display:none"', 'about:blank', esc_attr(Labels::getLocalized('dashboard_image_open')),
					'about:blank', esc_attr(Labels::getLocalized('dashboard_image_remove')));
				$imagesIds = array();
				foreach ($route->getImages() as $image):
					$imagesIds[] = $image->getId();
					printf($template,
						$image->getId(),
						'',
						esc_attr($image->isImage() ? $image->getImageUrl(Attachment::IMAGE_SIZE_FULL) : $image->getUrl()),
						esc_attr(Labels::getLocalized('dashboard_image_open')),
						esc_attr($image->getImageUrl(Attachment::IMAGE_SIZE_THUMB)),
						esc_attr(Labels::getLocalized('dashboard_image_remove'))
					);
				endforeach;
				?>
			</ul>
			<div class="cmloc-field-desc"<?php if (empty($imagesIds)) echo ' style="display:none;"'; ?>><?php echo Labels::getLocalized('dashboard_images_description'); ?></div>
			<div class="cmloc-images-add">
				<input type="hidden" name="images" value="<?php echo esc_attr(implode(',', $imagesIds)); ?>" />
				<span><?php echo Labels::getLocalized('dashboard_upload_image'); ?></span>
				<?php
				$accept = '';
				if($location_form_images == 'optional') { $accept .= ',image/*'; }
				if($location_form_videos == 'optional') { $accept .= ',video/*'; }
				$accept = substr($accept, 1);
				?>
				<input type="file" class="cmloc-images-upload" accept="<?php echo $accept; ?>" multiple />
				<?php
				if($location_form_videos == 'optional') {
					?>
					<div class="cmloc-btn cmloc-add-video-btn"><?php echo Labels::getLocalized('dashboard_video_add_btn'); ?></div>
					<?php
				}
				?>
			</div>
		</div>
		<?php } ?>
		
		<div id="cmloc-editor-map">
			<ul class="cmloc-inline-nav cmloc-toolbar">
				<li>
					<input type="text" class="cmloc-find-location" placeholder="<?php echo esc_attr(Labels::getLocalized('dashboard_map_search')); ?>" />
					<img id="get_my_current_location" src="<?php echo App::url('asset/img/currloc.png'); ?>" alt="<?php echo esc_attr(Labels::getLocalized('dashboard_current_loction')); ?>" title="<?php echo esc_attr(Labels::getLocalized('dashboard_current_loction')); ?>" />
				</li>
			</ul>
			<div id="cmloc-editor-map-canvas"></div>
		</div>
		
		<div id="cmloc-editor-locations">
			<ul class="cmloc-locations-list">
				<li data-id="0" style="display:none">
					<input class="location-id" type="hidden" name="locations[id][]" value="0" />
					<input class="location-name" type="hidden" name="locations[name][]" value="" placeholder="<?php echo esc_attr(Labels::getLocalized('location_name')); ?>" />
					
					<?php if ($location_form_latitude != 'none') { ?>
					<div class="cmloc-field cmloc-field-location-latitude"><label><?php echo Labels::getLocalized('location_latitude'); ?>:</label><input class="location-lat" type="text" name="locations[lat][]" value="" placeholder="<?php echo esc_attr(Labels::getLocalized('location_latitude')); ?>" /></div>
					<?php } else { ?>
					<input class="location-lat" type="hidden" name="locations[lat][]" value="" />
					<?php } ?>

					<?php if ($location_form_longitude != 'none') { ?>
					<div class="cmloc-field cmloc-field-location-longitude"><label><?php echo Labels::getLocalized('location_longitude'); ?>:</label><input class="location-long" type="text" name="locations[long][]" value="" placeholder="<?php echo esc_attr(Labels::getLocalized('location_longitude')); ?>" /></div>
					<?php } else { ?>
					<input class="location-long" type="hidden" name="locations[long][]" value="" />
					<?php } ?>

					<?php if ($location_form_markeralt != 'none') { ?>
					<div class="cmloc-field cmloc-field-location-marker-alt"><label><?php echo Labels::getLocalized('location_marker_alt'); ?>:</label><input class="location-marker-alt" type="text" name="locations[marker-alt][]" value="" title="<?php echo esc_attr(Labels::getLocalized('location_marker_alt')); ?>" <?php echo ($location_form_markeralt == 'required')?'required="required"':''; ?> /></div>
					<?php } ?>
					
					<?php if ($location_form_address != 'none') { ?>
					<div class="cmloc-field cmloc-field-location-address"><label><?php echo Labels::getLocalized('location_address'); ?>:</label><input class="location-address" type="text" name="locations[address][]" value="" title="<?php echo esc_attr(Labels::getLocalized('location_address')); ?>" <?php echo ($location_form_address == 'required')?'required="required"':''; ?> /></div>
					<?php } ?>
					
					<?php if ($location_form_postalcode != 'none') { ?>
					<div class="cmloc-field cmloc-field-location-postal"><label><?php echo Labels::getLocalized('location_postal_code'); ?>:</label><input class="location-postal-code" type="text" name="locations[postal-code][]" value="" title="<?php echo esc_attr(Labels::getLocalized('location_postal_code')); ?>" <?php echo ($location_form_postalcode == 'required')?'required="required"':''; ?> /></div>
					<?php } ?>

					<?php if (App::isPro()): ?>
						
						<?php if ($location_form_phonenumber != 'none') { ?>
						<div class="cmloc-field cmloc-field-location-phone"><label><?php echo Labels::getLocalized('location_phone_number'); ?>:</label><input class="location-phone-number" type="text" name="locations[phone-number][]" value="" title="<?php echo esc_attr(Labels::getLocalized('location_phone_number')); ?>" <?php echo ($location_form_phonenumber == 'required')?'required="required"':''; ?> /></div>
						<?php } ?>
						
						<?php if ($location_form_website != 'none') { ?>
						<div class="cmloc-field cmloc-field-location-website"><label><?php echo Labels::getLocalized('location_website'); ?>:</label><input class="location-website" type="text" name="locations[website][]" value="" title="<?php echo esc_attr(Labels::getLocalized('location_website')); ?>" <?php echo ($location_form_website == 'required')?'required="required"':''; ?> /></div>
						<?php } ?>
						
						<?php if ($location_form_emailaddress != 'none') { ?>
						<div class="cmloc-field cmloc-field-location-email"><label><?php echo Labels::getLocalized('location_email'); ?>:</label><input class="location-email" type="text" name="locations[email][]" value="" title="<?php echo esc_attr(Labels::getLocalized('location_email')); ?>" <?php echo ($location_form_emailaddress == 'required')?'required="required"':''; ?> /></div>
						<?php } ?>
						
						<?php if ($location_form_url != 'none') { ?>
						<div class="cmloc-field cmloc-field-location-url"><label><?php echo Labels::getLocalized('location_url'); ?>:</label><input class="location-url" type="text" name="locations[url][]" value="" title="<?php echo esc_attr(Labels::getLocalized('location_url')); ?>" <?php echo ($location_form_url == 'required')?'required="required"':''; ?> /></div>
						<?php } ?>

					<?php endif; ?>

					<input class="location-type" type="hidden" name="locations[type][]" value="location" />
					<div style="display:none"><textarea class="location-description" name="locations[description][]" placeholder="<?php echo esc_attr(Labels::getLocalized('location_description')); ?>"></textarea></div>
					<?php
					/*
					<div class="cmloc-images">
						<input type="hidden" name="locations[images][]" value="" />
						<ul class="cmloc-images-list">
							<li data-id="0" style="display:none"><a href="about:blank" target="_blank" title="<?php
								echo esc_attr(Labels::getLocalized('dashboard_image_open')); ?>"><img src="about:blank" alt="Image" /></a>
							<span class="cmloc-image-delete" title="<?php
								echo esc_attr(Labels::getLocalized('dashboard_image_remove')); ?>">&times;</span></li>
						</ul>
						<a href="#" class="button cmloc-images-add-btn"><?php
								echo Labels::getLocalized('dashboard_image_add'); ?></a>
					</div>
					*/
					?>
					<?php do_action('cmloc_route_editor_location_bottom', $route); ?>
                </li>
            </ul>
			<script type="text/javascript">
			jQuery(function($) {
				new CMLOC_Editor(<?php echo json_encode($route->getJSLocations()); ?>);
				$('.cmloc-images').each(CMLOC_Editor_Images_init);
				<?php do_action('cmloc_editor_wp_footer_js', $route); ?>
			});
			</script>
		</div>
		
		<?php echo apply_filters( 'cmloc_editor_before_form_summary', '' ); ?>
		
		<?php if ($location_form_terms != 'none' && !$route->getId()) { ?>
		<div class="form-terms">
			<input type="checkbox" name="terms-conditions" required />
			<?php echo Labels::getLocalized('location_terms'); ?>
		</div>
		<?php } ?>
		
		<div class="form-summary">
			<input type="hidden" name="<?php echo esc_attr(controller\DashboardController::EDITOR_NONCE); ?>" value="<?php echo esc_attr($nonce); ?>" class="cmloc-nonce" />
			<input type="submit" name="btn_save" value="<?php echo esc_attr(Labels::getLocalized('dashboard_save_btn')); ?>" class="button button-primary addeditlocation" />
			<?php if($route->getId()) { ?>
			<input type="button" name="btn_remove" value="<?php echo esc_attr(Labels::getLocalized('dashboard_remove_btn')); ?>" class="button button-primary btn_remove" data-id="<?php echo $route->getId(); ?>" />
			<?php } ?>
		</div>

	</form>
	
	<?php
	if($route->getId()) {

	$link_share_enable = Settings::getOption(Settings::OPTION_LOCATION_FORM_LINKSHARING);
	$link_share_page_id = Settings::getOption(Settings::OPTION_LINK_SHARE_PAGE_ID);

	$post = get_post($route->getId());
	
	$location_link_url = '';
	$location_category_link_url = '';
	if($link_share_page_id) {
		
		$location_link_url = get_the_permalink($link_share_page_id).'?location='.$post->post_name;

		$categories_id = '';
		$categories = $route->getCategories();
		if(count($categories) > 0) {
			foreach ($categories as $item) {
				$categories_id .= ','.$item->getSlug();
			}
			$categories_id = substr($categories_id, 1);
		}
		if($categories_id) {
			$location_category_link_url = get_the_permalink($link_share_page_id).'?category='.$categories_id;
		} else {
			$location_category_link_url = get_the_permalink($link_share_page_id);
		}
	}
	?>
	<?php if($link_share_enable == 'optional') { ?>
	<div class="location_share_link_box">
		<h6><?php echo Labels::getLocalized('location_link_sharing'); ?></h6>
		<div class="location_share_link_row location_share_link">
			<input type="text" value="<?php echo $location_link_url; ?>" id="cmloc_location_input" readonly="readonly" />
			<button onclick="cmloc_location_link()" id="cmloc_location_button"><?php echo Labels::getLocalized('location_copy_location'); ?></button>
		</div>
		<div class="location_share_link_row location_category_share_link">
			<input type="text" value="<?php echo $location_category_link_url; ?>" id="cmloc_location_category_input" readonly="readonly" />
			<button onclick="cmloc_location_category_link()" id="cmloc_location_category_button"><?php echo Labels::getLocalized('location_copy_location_category'); ?></button>
		</div>
	</div>
	<script type="text/javascript">
	function cmloc_location_link() {
		var copyText = document.getElementById("cmloc_location_input");
		copyText.select();
		copyText.setSelectionRange(0, 99999);
		document.execCommand("copy");
		document.getElementById("cmloc_location_button").innerHTML = '<?php echo Labels::getLocalized("location_copied"); ?>';
		setTimeout(function(){ document.getElementById("cmloc_location_button").innerHTML = '<?php echo Labels::getLocalized("location_copy_location"); ?>'; }, 3000);
	} 
	function cmloc_location_category_link() {
		var copyText = document.getElementById("cmloc_location_category_input");
		copyText.select();
		copyText.setSelectionRange(0, 99999);
		document.execCommand("copy");
		document.getElementById("cmloc_location_category_button").innerHTML = '<?php echo Labels::getLocalized("location_copied"); ?>';
		setTimeout(function(){ document.getElementById("cmloc_location_category_button").innerHTML = '<?php echo Labels::getLocalized("location_copy_location_category"); ?>'; }, 3000);
	} 
	</script>
	<?php }} ?>

	<?php echo apply_filters( 'cmloc_add_location_below_form', '' ); ?>

    <?php if( $route->getId() && current_user_can('manage_options') ) {
        if( Settings::getOption(Settings::OPTION_SHOW_SHORTCODES_ROUTE_EDIT_FRONTEND) ) { ?>
            <div class="cmloc-field cmloc-field-location-metabox-shortcodes">
                <label><?php echo Labels::getLocalized('shortcodes'); ?>:</label>
                <?php echo controller\RouteController::loadBackendView('metabox-shortcodes', array('id' => $route->getId())); ?>
            </div>
            <?php
        }
    } ?>

</div>